# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 7849 $
$LastChangedDate:: 2024-02-26 17:25:55 +0100#$

This file is part of SyMSpace.

Copyright (c) 2023, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import json

import yaml
import os
from symspace.osutils.pathutils import Paths
from symspace.wrapper.symwrapper import SymProject, SymField
from componentspace.base_component import Component
from componentspace.utils.component_filter import ComponentFilter


class Workflow:
    """
    Use this class to build a workflow component

    YAML file is needed
    """
    def __init__(self, filename: str):
        if not os.path.exists(filename):
            raise FileNotFoundError(f'File {filename} does not exist')

        with open(filename) as stream:
            if yaml.__version__ > '6.0':
                self.settings = yaml.load(stream, Loader=yaml.FullLoader)
            else:
                self.settings = yaml.load(stream)

        template = os.path.join(Paths.getComponentSpaceRoot(), '_template', 'Template.mop')
        if not os.path.exists(template):
            raise FileNotFoundError(f'File {template} does not exist')

        self.temp_project = SymProject.load(template)
        self.path = os.path.dirname(filename)
        self.name = self.settings['Info']['Name']

    def build(self):
        """
        Build component

        """

        # Assign values
        info = self.settings.get('Info', {})
        prj_info = self.temp_project['Info']
        prj_info['Name'].setData(info.get('Name', ''))
        prj_info['FunctionName'].setData(info.get('FunctionName', ''))
        prj_info['DisplayName'].setData(info.get('DisplayName', ''))
        prj_info['Type'].setData(info.get('Type', 'Component'))
        prj_info['Author'].setData(info.get('Author', ''))
        prj_info['OwnerDomain'].setData(info.get('OwnerDomain', 'lcm.at'))
        prj_info['Version'].setData(info.get('Version', '0.1'))
        prj_info['WebPath'].setData(info.get('WebPath', ''))
        prj_info['Tags'].setData(info.get('Tags', ''))

        flt = self.settings.get('Filter')
        self.temp_project['Filter.User'].setData(flt.get('User', None))
        self.temp_project['Filter.UserGroup'].setData(flt.get('UserGroup', None))
        if 'Workflow' in flt:
            param_wf = SymField('Workflow', 'Contains workflow as JSON string')
            self.temp_project['Filter'].addParam(param_wf)
            param_wf.setData(json.dumps(flt['Workflow']))

        # Add provides
        if 'provides' in flt:
            filter = ComponentFilter(self.temp_project)
            for item in flt['provides']:
                prov = flt['provides'][item]
                filter.addProvides(prov['value'], prov.get('displayName', None), prov.get('guid', ''))

        self.temp_project.saveAs(os.path.join(self.path, self.name + '.mop'), override=True)

        comp = Component(os.path.join(self.path, self.name + '.mop'))
        comp.buildComponent()
