# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 5114 $
$LastChangedDate:: 2021-10-25 13:12:32 +0200#$

This file is part of SyMSpace.

Copyright (c) 2021, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
import glob
import shutil
import time
import tempfile
import json
from abc import abstractmethod

from symspace.osutils.pathutils import Paths
from symspace.osutils.jenkins_utils import isBuildServer
from symspace.osutils.multiprocessing import ExternalMultiprocessing

import componentspace.osutils.fileutils as fu
from componentspace.dxf_geometry_component import DXFGeometryComponent
from componentspace.builders.scripts import build_external_geometry


class GeometryBatchComponent(DXFGeometryComponent):
    def __init__(self, comp_file):
        super().__init__(comp_file)
        self.__template_comp_file = comp_file

    def _getSingleCompFile(self, geometry: dict) -> str:
        """
        Return the path to the component project file

        Parameters
        ----------
        geometry : dict
            Dictionary of geometry properties

        Returns
        -------
        comp_file : str
        """
        component_name = geometry['component_name'] + "_DXF"
        # base_component = geometry['base_component']

        # Todo: this is just a workaround due to Windows Path 260 char limitations and Jep
        if isBuildServer():
            path_geometry = os.path.join(tempfile.gettempdir(), component_name)
        else:
            path_geometry = os.path.join(self.path, component_name)  # os.path.join(self.base_directory, component_name)

        comp_file = os.path.join(path_geometry, component_name + '.mop')
        # base_file = os.path.join(path_geometry, component_name + '.mop')
        return comp_file  # , base_file

    def _createSingleDirectory(self, geometry: dict) -> str:
        """
        Creates a directory with the name of the geometry and the component class
        Copies directories /rst, /help, and /doc

        Parameters
        ----------
        geometry : dict
            Dictionary of geometry properties

        Returns
        -------
        comp_file : str
        """
        comp_file = self._getSingleCompFile(geometry)
        path_geometry = os.path.dirname(comp_file)
        component_name = geometry['component_name']

        if os.path.exists(path_geometry):
            '''Remove material directory'''
            shutil.rmtree(path_geometry, onerror=fu.on_rm_error)
            '''wait until OS is finished'''
            cnt = 0
            while os.path.exists(path_geometry):
                time.sleep(1)
                cnt = cnt + 1
                if cnt > 3:
                    raise ValueError('Cannot remove path!')

        '''Create geometry directory'''
        if not os.path.exists(os.path.dirname(path_geometry)):
            os.makedirs(os.path.dirname(path_geometry))

        if not os.path.exists(path_geometry):
            os.mkdir(path_geometry)

        # comp_file = os.path.join(path_geometry, component_name + '.mop')
        shutil.copy(self.__template_comp_file, comp_file)
        self._createSingleCompfileFromTemplate(comp_file, geometry)

        return comp_file

    # # @abstractmethod
    # def extractMaterialData(self):
    #     raise NotImplementedError

    @abstractmethod
    def _createSingleCompfileFromTemplate(self, comp_file: str, geometry: dict):
        raise NotImplementedError

    # def buildComponent(self, keep_mopc: bool = False) -> str:
    #     raise Exception('Please use the method buildComponentBatch() instead of buildComponent()!')
    #
    # def buildTest(self, add_component: bool = True, add_testparam: bool = True):
    #     raise Exception('Please use the method buildComponentTestBatch() instead of buildTest()!')
    #
    def buildComponentBatch(self):
        """Build all geometry components"""
        files = glob.glob(os.path.join(self.base_directory, 'resources', '*.json'))

        # '''set a remote debug breakpoint - (only available from PyCharm Professional Edition or Eclipse PyDev)'''
        # import pydevd_pycharm
        # pydevd_pycharm.settrace('localhost', port=21000, stdoutToServer=True, stderrToServer=True)

        n_geo = len(files)

        xmp = ExternalMultiprocessing(max_cpu_percent=75.0, max_ram_percent=75.0, stdout=None)
        DebugMode = not isBuildServer()

        for i, filename in enumerate(files):
            print('Build Geometry #{:d} / {:d} '.format(i + 1, n_geo))

            with open(filename, 'r') as fh:
                data = json.load(fh)

            comp_file = self._createSingleDirectory(data)

            if DebugMode:
                '''single internal process -> slow only for debugging'''
                comp = DXFGeometryComponent(comp_file)
                # comp.base_directory = self.base_directory
                comp.buildComponent()
                comp.sym_mdl.close()
            else:
                '''build with external processes for better/multicore cpu usage'''
                base_directory = self.base_directory
                script_py = build_external_geometry.__file__
                cmd_args = [Paths.getCSyMSpaceExe(), script_py, comp_file, base_directory]
                xmp.startProcess(cmd_args, wait_to_start=False, wait_to_finish=False)
                xmp.wait(cpu_measure_interval=0.5)

        '''wait until all processes are done'''
        xmp.waitForCompletion()

    # def buildComponentTestBatch(self):
    #     """Build tests for material components"""
    #     if self.__class__.__name__ in ['MaterialFerromagnetic', 'MaterialVoest', 'MaterialCogentPower']:
    #         # Todo: this is just a workaround! All subclasses should have the required json files in the "resources" dir
    #         files = glob.glob(os.path.join(self.path, 'src', '*.json'))
    #     else:
    #         files = glob.glob(os.path.join(self.path, 'resources', '*.json'))
    #     n_mat = len(files)
    #
    #     for i, filename in enumerate(files):
    #         print('Build Material Test #{:d} / {:d} '.format(i + 1, n_mat))
    #
    #         with open(filename, 'r') as fh:
    #             material = json.load(fh)
    #
    #         comp_file = self._getSingleCompFile(material)
    #         comp = MaterialComponent(comp_file)
    #         comp.base_directory = self.path
    #         comp.buildTest()
