#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 5787 $
$LastChangedDate:: 2022-04-21 11:40:05 +0200#$

This file is part of SyMSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import os
import glob
import stat
import fileinput
import fnmatch
from symspace.osutils.shutil_wrapper import ShutilWrapper


def searchBuildComponentScripts(start_path: str, exclude_dirs: tuple = ('_template',),
                                file_name: str = 'build_component.py') -> list:
    file_paths = []
    for file in glob.iglob(start_path + '/**/{}'.format(file_name), recursive=True):
        parent_dir = os.path.basename(os.path.dirname(file))
        if parent_dir not in exclude_dirs:
            file_paths.append(file)
    return file_paths


def copyTree(src: str, dst: str, symlinks: bool = False, ignore=None) -> list:
    """
    Copies directory with files recursively

    Parameters
    ----------
    src : str
        Source path
    dst : str
        Destination path
    symlinks : bool
    ignore

    Returns
    -------
    filelist : list
        List of copied files
    """
    filelist = []

    if not os.path.exists(dst):
        os.makedirs(dst)
    for item in os.listdir(src):
        # Skip Subversion directories
        if item == '.svn':
            continue
        # Skip __pycache__ directory
        if item == '__pycache__':
            continue
        s = os.path.join(src, item)
        d = os.path.join(dst, item)
        if os.path.isdir(s):
            lst = copyTree(s, d, symlinks, ignore)
            if lst:
                filelist.extend(lst)
        else:
            if not os.path.exists(d) or os.stat(s).st_mtime - os.stat(d).st_mtime > 1:
                ShutilWrapper.copy2(s, d)
                os.chmod(d, stat.S_IWRITE)
                filelist.append(d)
                # print(d, os.access(d, os.W_OK))

    return filelist


def copyFile(src: str, dst: str) -> str:
    """
    Copies files and removes readonly

    Parameters
    ----------
    src : str
        Source path
    dst : str
        Destination path

    Returns
    -------
    dst_file : str
        Destination file path
    """

    if not os.path.exists(dst):
        os.makedirs(dst)

    dst_file = ShutilWrapper.copy2(src, dst)
    os.chmod(dst_file, stat.S_IWRITE)
    return dst_file


def copyFileAndReplace(infile: str, outfile: str, replace: list, encoding: str = 'utf-8'):
    """
    Copy text file and replace content

    Parameters
    ----------
    infile : string
    outfile : string
    replace : list of tuples
    encoding: str
        Result file encoding, utf-8 (Default)
    """
    py_src = ''
    with open(infile, 'r') as file:
        for line in file:
            py_src = py_src + line
    for r in replace:
        py_src = py_src.replace(r[0], r[1])

    with open(outfile, 'w+', encoding=encoding) as file:
        file.write(py_src)


def replaceInFile(file: str, old: str, new: str):
    """Replace text in file

    Parameters
    ----------
    file : str
        File name
    old : str
        Text to be replaced
    new : str
        Text replaced
    """
    with fileinput.FileInput(file, inplace=True) as f:
        for line in f:
            print(line.replace(old, new), end='')


def on_rm_error(func, path, exc_info):
    # path contains the path of the file that couldn't be removed
    # let's just assume that it's read-only and unlink it.
    os.chmod(path, stat.S_IWRITE)
    os.unlink(path)


def includePatterns(*patterns):
    """ Function that can be used as shutil.copytree() ignore parameter that
    determines which files *not* to ignore, the inverse of "normal" usage.

    This is a factory function that creates a function which can be used as a
    callable for copytree()'s ignore argument, *not* ignoring files that match
    any of the glob-style patterns provided.

    ‛patterns’ are a sequence of pattern strings used to identify the files to
    include when copying the directory tree.

    Example usage:

        copytree(src_directory, dst_directory,
                 ignore=include_patterns('*.sldasm', '*.sldprt'))
    """

    def _ignore_patterns(path, all_names):
        # Determine names which match one or more patterns (that shouldn't be
        # ignored).
        keep = (name for pattern in patterns
                for name in fnmatch.filter(all_names, pattern))
        # Ignore file names which *didn't* match any of the patterns given that
        # aren't directory names.
        dir_names = (name for name in all_names if os.path.isdir(os.path.join(path, name)))
        return set(all_names) - set(keep) - set(dir_names)

    return _ignore_patterns


def isWinLongPath(path, max_len=250):
    """Check if the path might be restricted by windows path lenght limitations

      By default Windows is restricted to paths with max 260 chars
    """
    if os.name == 'nt' and len(path) > max_len:
        try:
            '''check if the restriction is lifted by a registry entry (available in Windows 10)'''
            import winreg
            path = r'SYSTEM\CurrentControlSet\Control\FileSystem'
            key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, path)
            value = winreg.QueryValueEx(key, "LongPathsEnabled")[0]
            ret = not bool(value)
        except Exception:
            ret = True
        return ret
    else:
        return False
